
!SocketStream class methodsFor: '*UnixSockets-instance creation' stamp: 'tonyg 6/7/2022 10:12'!
openLocalStreamPath: path
	"Creates a UNIX-domain ('local') socket in SOCK_STREAM mode, and connects it to `path`."
	| s a |
	s := Socket newStream: SocketAddressInformation addressFamilyLocal.
	a := SocketAddressInformation forLocalPath: path socketType: SocketAddressInformation socketTypeStream.
	s connectTo: a first socketAddress.
	^ self on: s! !

!SocketAddressInformation class methodsFor: '*UnixSockets-instance creation' stamp: 'tonyg 6/7/2022 10:10'!
forLocalPath: path socketType: socketType
	"e.g. SocketAddressInformation forLocalPath: '/path/to/socket' socketType: SocketAddressInformation socketTypeStream"
	^ self
		forHost: ''
		service: path
		flags: 0
		addressFamily: self addressFamilyLocal
		socketType: socketType
		protocol: self protocolUnspecified! !

!Socket class methodsFor: '*UnixSockets-instance creation' stamp: 'tonyg 6/7/2022 09:57'!
newDatagram: family
	"Create a socket and initialise it for datagram transfer."

	"FIXME This method only exists to call initialize:withFamily: instead of the extant
	initialize:family:, since that latter ignores `family` if old-style networking is enabled.
	Once initialize:family: can be safely used, we can merge this with the extant newUDP:."
	
	self initializeNetwork.
	^[ super new initialize: UDPSocketType "sic" withFamily: family ]
		repeatWithGCIf: [ :socket | socket isValid not ]! !

!Socket class methodsFor: '*UnixSockets-instance creation' stamp: 'tonyg 6/7/2022 10:25'!
newDatagramLocal: path
	"Creates a UNIX-domain ('local') socket in SOCK_DGRAM mode, and connects it to `path`."
	| s a |
	s := self newDatagram: SocketAddressInformation addressFamilyLocal.
	a := SocketAddressInformation forLocalPath: path socketType: SocketAddressInformation socketTypeDGram.
	"Using the primitive directly is gross, but datagram sockets claim to be 'connected' when first created,
	so we use the primitive to bypass the sanity check in Socket>>connectNonBlockingTo:."
	s primSocket: s socketHandle connectTo: a first socketAddress.
	^ s! !

!Socket class methodsFor: '*UnixSockets-instance creation' stamp: 'tonyg 6/7/2022 09:57'!
newStream: family
	"Create a socket and initialise it for streaming."

	"FIXME This method only exists to call initialize:withFamily: instead of the extant
	initialize:family:, since that latter ignores `family` if old-style networking is enabled.
	Once initialize:family: can be safely used, we can merge this with the extant newTCP:."
	
	self initializeNetwork.
	^[ super new initialize: TCPSocketType "sic" withFamily: family ]
		repeatWithGCIf: [ :socket | socket isValid not ]! !

!Socket methodsFor: '*UnixSockets-initialize-destroy' stamp: 'tonyg 6/7/2022 09:59'!
initialize: socketType withFamily: addressFamily
	"Initialize a new socket handle. If socket creation fails, socketHandle will be set to nil."
	
	"FIXME. This method only exists because initialize:family: checks `NetNameResolver useOldNetwork`,
	and if it answers true, it ignores addressFamily. We want never to ignore addressFamily.
	If and when initialize:family: is changed, this method can be removed and senders redirected."

	family := addressFamily.
	self initializeSocketHandleUsing: [ :semaIndex :readSemaIndex :writeSemaIndex |
		self primSocketCreateNetwork: family
			type: socketType
			receiveBufferSize: DefaultReceiveBufferSize
			sendBufSize: DefaultSendBufferSize
			semaIndex: semaIndex
			readSemaIndex: readSemaIndex
			writeSemaIndex: writeSemaIndex ]! !
